"""
    Demo DynamicPolicy hasznlatra.
    Egy H2O instance allapot tulajdonsga "szilrd" vagy "folykony" lehet(ltrehozs utn "folykony").
    Az allapot tulajdonsg mindig lekrdezhet, de nem lehet direktben mdostani. A masolj metdus is
    mindig meghvhat s a visszadott rtk egy j H2O instance lesz az aktulis llapottal.
    A fagyjmeg s az olvadjmeg metdusok csak a megfelel allapot rtk esetn lteznek, azaz pl.
    "szilrd" allapot esetben fagyjmeg hvsa "Member not found." hibazenetet generl.
"""
import pythoncom
import winerror
import win32com.server.util
from win32com.server.policy import DynamicPolicy
from win32com.server.exception import COMException
import types


class H2O:
    """
        H2O demo class
    """
    #COM regisztrcis informcik
    _reg_desc_          = 'H2O demo class'
    _reg_clsid_         = '{3D80E5B1-E619-11D4-BF65-00C0DF0463FE}'
    _reg_progid_        = 'Demo.H2O'
    _reg_verprogid_     = 'Demo.H2O.1'
    _reg_policy_spec_   = 'DynamicPolicy'   # UseCommandLine regisztrci sorn innen tudja, hogy DynamicPolicy-t szeretnnk

    #Csak az egyszersg kedvrt a kt llapotban elrhet attributumokat/metdusokat
    #kigyjtttem kt listba
    _szilard_attrs_ = ["olvadjmeg","masolj","allapot"]
    _folyekony_attrs_ = ["fagyjmeg","masolj","allapot"]

    def olvadjmeg(self):
        """
            COM-on keresztl csak "szilrd" llapot esetn hvhat
        """
        self.allapot="folykony"

    def fagyjmeg(self):
        """
            COM-on keresztl csak "folykony" llapot esetn hvhat
        """
        self.allapot="szilrd"

    def masolj(self):
        """
            Ha egy Python class-t szeretnnk visszadni COM-on keresztl,
            akkor wrap hvsval tudunk belle egy IDispatch interfszt csinlni.
            Meg kell adni a policy-t is, mert az alaprtelmezett a DesignatedWrapPolicy.
        """
        return win32com.server.util.wrap(H2O(self.allapot),usePolicy=DynamicPolicy)
        
    def __init__(self,allapot="folykony"):
        """
            Alaprtelmezs szerint "folykony" lesz(kliens csak ilyet tud csinlni!),
            de a masolj-hoz hasonl metdusokban - ahol egy j pldnyt ad vissza
            a metdus - lehetsg van "szilrd"/"folykony" llapot pldny ltrehozsra.
        """
        self.allapot=allapot
    
    def _dynamic_(self, name, lcid, wFlags, args):
        #nzzk az adott llapotban megengedett attributumokra hivatkozik-e a kliens
        if (self.allapot=="szilrd" and name.lower() not in self._szilard_attrs_) or \
           (self.allapot=="folykony" and name.lower() not in self._folyekony_attrs_):
            raise COMException("%s nev attributum nem ltezik!" % name, winerror.DISP_E_MEMBERNOTFOUND)
        item = getattr(self,name.lower())   # hogy VB ne kavarjon be itt minden kisbets
        if wFlags & pythoncom.DISPATCH_METHOD and type(item) in [types.MethodType]:
            #metdus hvs
            return apply(item,args)
        elif wFlags & pythoncom.DISPATCH_PROPERTYGET:
            # Python kliens elszr ezzel prblkozik metdus esetben is
            # Az albbi exception generlsval brjuk r hogy metdusknt is
            # megprblja elrni(kicsit dinnye, de ez van)
            if type(item) in [types.MethodType]:
                raise COMException("%s metdus nem tulajdonsg!" % name, winerror.DISP_E_MEMBERNOTFOUND)
            return item
        elif wFlags & (pythoncom.DISPATCH_PROPERTYPUT | pythoncom.DISPATCH_PROPERTYPUTREF):
            #tulajdosnsg bellts(itt: allapot) nem engedlyezett
            raise COMException("rtkads %s tulajdonsgnak nem engedlyezett!" % name, winerror.E_ACCESSDENIED)
        else:
            raise COMException("invalid wFlags", winerror.E_INVALIDARG)

def Register():
    # szoksos regisztrls, de a policy nem az alaprtelmezett DesignatedWrapPolicy lesz,
    # hanem a H2O class _reg_policy_spec_ attributumban belltott DynamicPolicy
    from win32com.server.register import UseCommandLine
    return UseCommandLine(H2O)

if __name__=='__main__':
    Register()
